@@ -1,10 +1,10 @@
 import inspect
 import os
 
-from jug.subcommands import status
-from jug.tests.task_reset import task_reset
-from jug.tests.utils import simple_execute
 from jug.options import default_options
+from jug.subcommands import status
+from .task_reset import task_reset_at_exit, task_reset
+from .utils import simple_execute
 import jug
 
 
@@ -40,6 +40,19 @@ def test_cache():
     simple_execute()
     assert status.status(options) == 8 * 4
 
+@task_reset
+def test_cache_block_status():
+    jugfile = os.path.join(_jugdir, 'block_access.py')
+
+    options = default_options.copy()
+    options.jugdir = 'dict_store'
+    options.jugfile = jugfile
+    options.verbose = 'quiet'
+    options.status_cache = True
+    options.status_cache_file = ':memory:'
+
+    assert status.status(options) == 0
+
 @task_reset
 def test_cache_bvalue():
     jugfile = os.path.join(_jugdir, 'bvalue.py')
